<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <title>FMC Login</title>
</head>
<body>
    <h1>FMC Report</h1>
    <form action="{{ url_for('fmc_login') }}" method="POST">

            <label for="hostname">Hostname:</label><br>
            <input type="text" id="hostname" name="hostname" required><br>

            <label for="username">Username:</label><br>
            <input type="text" id="username" name="username" required><br>

            <label for="password">Password:</label><br>
            <input type="password" id="password" name="password" required><br>

            <input type="submit" id="submit" name="submit" value="Login">

        </form>
</body>
</html>